package ru.dbotthepony.mc.otm.datagen.blocks

import net.minecraft.resources.ResourceLocation
import net.minecraft.world.level.block.state.properties.BlockStateProperties
import net.minecraft.world.level.block.state.properties.DoubleBlockHalf
import net.minecraftforge.client.model.generators.ConfiguredModel
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.block.CableBlock
import ru.dbotthepony.mc.otm.block.CargoCrateBlock
import ru.dbotthepony.mc.otm.block.RotatableMatteryBlock
import ru.dbotthepony.mc.otm.block.entity.WorkerState
import ru.dbotthepony.mc.otm.block.matter.MatterBottlerBlock
import ru.dbotthepony.mc.otm.core.registryName
import ru.dbotthepony.mc.otm.datagen.modLocation
import ru.dbotthepony.mc.otm.datagen.toXRotBlockstate
import ru.dbotthepony.mc.otm.datagen.toXRotBlockstateInv
import ru.dbotthepony.mc.otm.datagen.toYRotBlockstate
import ru.dbotthepony.mc.otm.datagen.toYRotBlockstateInv
import ru.dbotthepony.mc.otm.registry.MBlocks
import ru.dbotthepony.mc.otm.registry.MNames
import ru.dbotthepony.mc.otm.registry.MRegistry

fun addBlockStates(provider: MatteryBlockStateProvider) {
	provider.block(MBlocks.BLACK_HOLE)
	provider.block(MBlocks.ANDROID_STATION)

	provider.ore(MBlocks.DEEPSLATE_TRITANIUM_ORE)
	provider.ore(MBlocks.TRITANIUM_ORE)
	provider.ore(MBlocks.TRITANIUM_RAW_BLOCK)

	provider.block(MBlocks.CHEMICAL_GENERATOR)
	provider.block(MBlocks.MATTER_SCANNER)
	provider.block(MBlocks.ITEM_MONITOR)

	provider.exec {
		with(provider.getMultipartBuilder(MBlocks.PHANTOM_ATTRACTOR)) {
			for (dir in RotatableMatteryBlock.FACING.possibleValues) {
				part().modelFile(provider.models().getExistingFile(modLocation("block/${MNames.PHANTOM_ATTRACTOR}")))
					.rotationY(dir.toYRotBlockstate())
					.addModel()
					.condition(BlockStateProperties.DOUBLE_BLOCK_HALF, DoubleBlockHalf.LOWER)
					.condition(RotatableMatteryBlock.FACING, dir)
					.end()
			}
		}

		with(provider.getMultipartBuilder(MBlocks.MATTER_BOTTLER)) {
			for (dir in RotatableMatteryBlock.FACING.possibleValues) {
				for (enum in WorkerState.SEMI_WORKER_STATE.possibleValues) {
					part().modelFile(provider.models().getExistingFile(ResourceLocation(OverdriveThatMatters.MOD_ID, "matter_bottler_${enum.name.lowercase()}")))
						.rotationY(dir.toYRotBlockstate())
						.addModel()
						.condition(RotatableMatteryBlock.FACING, dir)
						.condition(WorkerState.WORKER_STATE, enum)
						.end()
				}
			}

			for (dir in RotatableMatteryBlock.FACING.possibleValues) {
				for (enum in MatterBottlerBlock.SLOT_PROPERTIES) {
					part().modelFile(provider.models().getExistingFile(ResourceLocation(OverdriveThatMatters.MOD_ID, "matter_bottler_${enum.name}_open")))
						.rotationY(dir.toYRotBlockstate())
						.addModel()
						.condition(RotatableMatteryBlock.FACING, dir)
						.condition(enum, false)
						.end()

					part().modelFile(provider.models().getExistingFile(ResourceLocation(OverdriveThatMatters.MOD_ID, "matter_bottler_${enum.name}_closed")))
						.rotationY(dir.toYRotBlockstate())
						.addModel()
						.condition(RotatableMatteryBlock.FACING, dir)
						.condition(enum, true)
						.end()
				}
			}
		}
	}

	provider.block(MBlocks.MATTER_DECOMPOSER)
	provider.block(MBlocks.MATTER_REPLICATOR)
	provider.block(MBlocks.PLATE_PRESS)
	provider.block(MBlocks.GRAVITATION_STABILIZER)
	provider.block(MBlocks.GRAVITATION_STABILIZER_LENS)

	provider.block(MBlocks.STORAGE_POWER_SUPPLIER)
	provider.block(MBlocks.MATTER_RECYCLER)
	provider.block(MBlocks.ENERGY_SERVO)

	provider.exec {
		for (crate in MRegistry.CARGO_CRATES.allBlocks.values) {
			provider.getVariantBuilder(crate).forAllStates {
				return@forAllStates arrayOf(
					ConfiguredModel.builder()
						.modelFile(provider.models().getExistingFile(
							ResourceLocation(
								OverdriveThatMatters.MOD_ID, "${crate.registryName!!.path}_${if (it.getValue(
										CargoCrateBlock.IS_OPEN)) "open" else "closed"}")
						))
						.rotationY(it.getValue(RotatableMatteryBlock.FACING).toYRotBlockstate())
						.buildLast()
				)
			}
		}

		with(provider.getMultipartBuilder(MBlocks.STORAGE_BUS)) {
			for (dir in net.minecraft.core.Direction.values()) {
				part().modelFile(provider.models().getExistingFile(ResourceLocation(OverdriveThatMatters.MOD_ID, "storage_bus")))
					.rotationX(dir.toXRotBlockstate())
					.rotationY(dir.toYRotBlockstate())
					.addModel()
					.condition(RotatableMatteryBlock.FACING_FULL, dir)
					.end()

				part().modelFile(provider.models().getExistingFile(ResourceLocation(OverdriveThatMatters.MOD_ID, "storage_cable_connection")))
					.rotationX(dir.toXRotBlockstateInv())
					.rotationY(dir.toYRotBlockstateInv())
					.addModel()
					.condition(CableBlock.MAPPING_CONNECTION_PROP[dir.ordinal], true)
					.end()
			}

			part().modelFile(provider.models().getExistingFile(ResourceLocation(OverdriveThatMatters.MOD_ID, "storage_cable_core")))
				.addModel().end()
		}

		with(provider.getMultipartBuilder(MBlocks.STORAGE_IMPORTER)) {
			for (dir in net.minecraft.core.Direction.values()) {
				part().modelFile(provider.models().getExistingFile(ResourceLocation(OverdriveThatMatters.MOD_ID, "storage_importer")))
					.rotationX(dir.toXRotBlockstate())
					.rotationY(dir.toYRotBlockstate())
					.addModel()
					.condition(RotatableMatteryBlock.FACING_FULL, dir)
					.end()

				part().modelFile(provider.models().getExistingFile(ResourceLocation(OverdriveThatMatters.MOD_ID, "storage_cable_connection")))
					.rotationX(dir.toXRotBlockstateInv())
					.rotationY(dir.toYRotBlockstateInv())
					.addModel()
					.condition(CableBlock.MAPPING_CONNECTION_PROP[dir.ordinal], true)
					.end()
			}

			part().modelFile(provider.models().getExistingFile(ResourceLocation(OverdriveThatMatters.MOD_ID, "storage_cable_core")))
				.addModel().end()
		}

		with(provider.getMultipartBuilder(MBlocks.STORAGE_EXPORTER)) {
			for (dir in net.minecraft.core.Direction.values()) {
				part().modelFile(provider.models().getExistingFile(ResourceLocation(OverdriveThatMatters.MOD_ID, "storage_exporter")))
					.rotationX(dir.toXRotBlockstate())
					.rotationY(dir.toYRotBlockstate())
					.addModel()
					.condition(RotatableMatteryBlock.FACING_FULL, dir)
					.end()

				part().modelFile(provider.models().getExistingFile(ResourceLocation(OverdriveThatMatters.MOD_ID, "storage_cable_connection")))
					.rotationX(dir.toXRotBlockstateInv())
					.rotationY(dir.toYRotBlockstateInv())
					.addModel()
					.condition(CableBlock.MAPPING_CONNECTION_PROP[dir.ordinal], true)
					.end()
			}

			part().modelFile(provider.models().getExistingFile(ResourceLocation(OverdriveThatMatters.MOD_ID, "storage_cable_core")))
				.addModel().end()
		}
	}
}
