package ru.dbotthepony.mc.otm.datagen.loot

import net.minecraft.resources.ResourceLocation
import net.minecraft.world.entity.EntityType
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.storage.loot.predicates.LootItemCondition
import net.minecraft.world.level.storage.loot.predicates.LootItemKilledByPlayerCondition
import net.minecraftforge.common.loot.LootTableIdCondition
import ru.dbotthepony.mc.otm.data.ChanceWithPlaytimeCondition
import ru.dbotthepony.mc.otm.data.HasExosuitCondition
import ru.dbotthepony.mc.otm.data.ItemInInventoryCondition
import ru.dbotthepony.mc.otm.data.KilledByRealPlayer
import ru.dbotthepony.mc.otm.data.LootTableBasicAppender
import ru.dbotthepony.mc.otm.data.LootTableSeparatedAppender
import ru.dbotthepony.mc.otm.registry.MItems

fun LootTableIdCondition(location: String): LootItemCondition {
	return LootTableIdCondition.Builder(ResourceLocation("minecraft", location)).build()
}

fun LootTableIdCondition(location: ResourceLocation): LootItemCondition {
	return LootTableIdCondition.Builder(location).build()
}

fun addLootModifiers(it: LootModifiers) {
	it.add("dungeon_pill", LootTableSeparatedAppender(
		arrayOf(LootTableIdCondition.Builder(ResourceLocation("chests/simple_dungeon")).build()),
		ItemStack(MItems.PILL_ANDROID, 1) to 0.4,
		ItemStack(MItems.PILL_HEAL, 2) to 0.5,
		ItemStack(MItems.PILL_HEAL, 1) to 0.75,
	))

	it.add("mineshaft_pill", LootTableSeparatedAppender(
		arrayOf(LootTableIdCondition.Builder(ResourceLocation("chests/abandoned_mineshaft")).build()),
		ItemStack(MItems.PILL_ANDROID, 1) to 0.075,
		ItemStack(MItems.PILL_HEAL, 2) to 0.1,
		ItemStack(MItems.PILL_HEAL, 1) to 0.4,
	))

	it.add("mineshaft_nutrient_paste", LootTableSeparatedAppender(
		arrayOf(LootTableIdCondition.Builder(ResourceLocation("chests/abandoned_mineshaft")).build()),
		ItemStack(MItems.NUTRIENT_PASTE, 6) to 0.5,
		ItemStack(MItems.NUTRIENT_PASTE, 8) to 0.35,
		ItemStack(MItems.NUTRIENT_PASTE, 12) to 0.15,
	))

	it.add("desert_pyramid_pill", LootTableSeparatedAppender(
		arrayOf(LootTableIdCondition.Builder(ResourceLocation("chests/abandoned_mineshaft")).build()),
		ItemStack(MItems.PILL_ANDROID, 1) to 0.05,
		ItemStack(MItems.PILL_HEAL, 1) to 0.3,
	))

	it.add("jungle_temple_pill", LootTableSeparatedAppender(
		arrayOf(LootTableIdCondition.Builder(ResourceLocation("chests/jungle_temple")).build()),
		ItemStack(MItems.PILL_ANDROID, 1) to 0.5
	))

	it.add("end_city_modifications", LootTableSeparatedAppender(
		arrayOf(LootTableIdCondition.Builder(ResourceLocation("chests/end_city_treasure")).build()),
		ItemStack(MItems.PILL_ANDROID, 1) to 0.1,
		ItemStack(MItems.PILL_HUMANE, 1) to 0.3,
		ItemStack(MItems.PILL_OBLIVION, 1) to 0.5,
		ItemStack(MItems.ZPM_BATTERY, 1) to 0.005,
	))

	it.add("shipwreck_supply_pill", LootTableSeparatedAppender(
		arrayOf(LootTableIdCondition.Builder(ResourceLocation("chests/shipwreck_supply")).build()),
		ItemStack(MItems.PILL_HUMANE, 1) to 0.4,
		ItemStack(MItems.PILL_HEAL, 1) to 0.6,
	))

	it.add("shipwreck_supply_nutrient_paste", LootTableSeparatedAppender(
		arrayOf(LootTableIdCondition.Builder(ResourceLocation("chests/shipwreck_supply")).build()),
		ItemStack(MItems.NUTRIENT_PASTE, 12) to 0.85,
		ItemStack(MItems.NUTRIENT_PASTE, 24) to 0.35,
		ItemStack(MItems.NUTRIENT_PASTE, 38) to 0.1,
	))

	it.add("exosuit_probe_enderman_drop", LootTableBasicAppender(
		arrayOf(
			LootTableIdCondition(EntityType.ENDERMAN.defaultLootTable),
			HasExosuitCondition.INVERTED,
			KilledByRealPlayer,
			ChanceWithPlaytimeCondition(
				minPlaytime = 20 * 60 * 10,
				maxPlaytime = 20 * 60 * 120,
				minProbability = 0.1,
				maxProbability = 0.5,
			),
			ItemInInventoryCondition(
				ItemStack(MItems.EXOSUIT_PROBE)
			).invert().build()
		),
		listOf(
			ItemStack(MItems.EXOSUIT_PROBE)
		)
	))
}
