package ru.dbotthepony.mc.otm.android

import it.unimi.dsi.fastutil.objects.ObjectArraySet
import net.minecraft.data.CachedOutput
import net.minecraft.data.DataGenerator
import net.minecraft.data.DataProvider
import net.minecraft.resources.ResourceLocation
import java.util.LinkedList
import java.util.function.Consumer

@Suppress("unused")
open class AndroidResearchDataProvider(protected val dataGenerator: DataGenerator) : DataProvider {
	protected val pathProvider: DataGenerator.PathProvider = dataGenerator.createPathProvider(DataGenerator.Target.DATA_PACK, AndroidResearchManager.DIRECTORY)

	protected val callbacks = LinkedList<(Consumer<AndroidResearchType>) -> Unit>()

	/**
	 * override this
	 */
	protected open fun addEverything(serializer: Consumer<AndroidResearchType>) {
		for (callback in callbacks) {
			callback.invoke(serializer)
		}
	}

	/**
	 * or call this
	 */
	fun exec(callback: (Consumer<AndroidResearchType>) -> Unit): AndroidResearchDataProvider {
		callbacks.add(callback)
		return this
	}

	final override fun run(output: CachedOutput) {
		AndroidResearchManager.fireRegistrationEvent()

		val set = ObjectArraySet<ResourceLocation>()
		val added = LinkedList<AndroidResearchType>()

		addEverything {
			if (set.add(it.id)) {
				DataProvider.saveStable(output, it.toJson(), pathProvider.json(it.id))
				AndroidResearchManager.put(it)
				added.add(it)
			} else {
				throw IllegalStateException("Duplicate android research with ID ${it.id}")
			}
		}

		for (value in added) {
			value.validate()
		}
	}

	override fun getName(): String {
		return "Android Research"
	}
}
