package ru.dbotthepony.mc.otm.block

import net.minecraft.core.BlockPos
import net.minecraft.world.Containers
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.item.DyeColor
import net.minecraft.world.item.context.BlockPlaceContext
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.EntityBlock
import net.minecraft.world.level.block.entity.BlockEntity
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.level.block.state.StateDefinition
import net.minecraft.world.level.block.state.properties.BooleanProperty
import net.minecraft.world.level.material.Material
import net.minecraft.world.level.material.MaterialColor
import ru.dbotthepony.mc.otm.block.entity.CargoCrateBlockEntity

class CargoCrateBlock(val color: DyeColor?) : RotatableMatteryBlock(
	Properties.of(Material.STONE, color?.materialColor ?: MaterialColor.COLOR_BLUE).requiresCorrectToolForDrops().strength(1.5f, 30.0f)
), EntityBlock {
	override fun newBlockEntity(blockPos: BlockPos, blockState: BlockState): BlockEntity {
		return CargoCrateBlockEntity(blockPos, blockState)
	}

	override fun createBlockStateDefinition(builder: StateDefinition.Builder<Block, BlockState>) {
		super.createBlockStateDefinition(builder)
		builder.add(IS_OPEN)
	}

	override fun getStateForPlacement(context: BlockPlaceContext): BlockState? {
		return super.getStateForPlacement(context)?.setValue(IS_OPEN, false)
	}

	override fun hasAnalogOutputSignal(p_60457_: BlockState): Boolean {
		return true
	}

	override fun getAnalogOutputSignal(p_60487_: BlockState, level: Level, p_60489_: BlockPos): Int {
		val tile = level.getBlockEntity(p_60489_) as? CargoCrateBlockEntity ?: return 0
		return AbstractContainerMenu.getRedstoneSignalFromContainer(tile.container)
	}

	companion object {
		@JvmField
		val IS_OPEN: BooleanProperty = BooleanProperty.create("open")
	}
}