package ru.dbotthepony.mc.otm.client

import com.mojang.blaze3d.platform.InputConstants
import net.minecraft.client.KeyMapping
import net.minecraftforge.client.event.RegisterKeyMappingsEvent
import net.minecraftforge.client.event.RenderLevelStageEvent
import net.minecraftforge.client.settings.KeyConflictContext
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.android.AndroidActiveFeature
import ru.dbotthepony.mc.otm.capability.matteryPlayer
import ru.dbotthepony.mc.otm.client.render.is3DContext
import ru.dbotthepony.mc.otm.network.ActivateAndroidFeaturePacket
import ru.dbotthepony.mc.otm.network.MatteryPlayerNetworkChannel

object AndroidAbilityKeyMapping : KeyMapping("key.otm.android_ability", KeyConflictContext.IN_GAME, InputConstants.Type.KEYSYM.getOrCreate(InputConstants.KEY_V), OverdriveThatMatters.MOD_ID) {
	private var feature: AndroidActiveFeature? = null

	override fun setDown(isDown: Boolean) {
		val old = this.isDown
		super.setDown(isDown)

		if (old != isDown) {
			if (isDown) {
				val capability = minecraft.player?.matteryPlayer ?: return

				if (capability.isAndroid) {
					feature = capability.features.map { it as? AndroidActiveFeature }.filter { it != null && it.isActive }.findFirst().orElse(null)
				} else {
					feature = null
				}
			} else {
				val feature = feature

				if (feature != null && feature.activate(true) && minecraft.player != null && feature.android.isAndroid) {
					MatteryPlayerNetworkChannel.sendToServer(ActivateAndroidFeaturePacket(feature.type))
				}

				this.feature = null
			}
		}
	}

	fun onRenderLevel(event: RenderLevelStageEvent) {
		if (!isDown) {
			return
		}

		val feature = feature ?: return

		if (event.stage === feature.previewRenderStage) {
			try {
				is3DContext = true
				event.poseStack.pushPose()
				feature.renderPreview(event)
			} finally {
				is3DContext = false
				event.poseStack.popPose()
			}
		}
	}

	fun register(event: RegisterKeyMappingsEvent) {
		event.register(this)
	}
}
