package ru.dbotthepony.mc.otm.client.screen.widget

import com.mojang.blaze3d.systems.RenderSystem
import com.mojang.blaze3d.vertex.BufferUploader
import com.mojang.blaze3d.vertex.DefaultVertexFormat
import com.mojang.blaze3d.vertex.PoseStack
import com.mojang.blaze3d.vertex.VertexFormat
import net.minecraft.client.gui.screens.Screen
import net.minecraft.client.renderer.GameRenderer
import net.minecraft.network.chat.Component
import org.lwjgl.opengl.GL11
import ru.dbotthepony.mc.otm.client.render.WidgetLocation
import ru.dbotthepony.mc.otm.client.render.element
import ru.dbotthepony.mc.otm.client.render.tesselator
import ru.dbotthepony.mc.otm.client.screen.panels.EditablePanel
import ru.dbotthepony.mc.otm.core.TranslatableComponent
import ru.dbotthepony.mc.otm.core.formatMatterLevel
import ru.dbotthepony.mc.otm.menu.widget.LevelGaugeWidget
import kotlin.math.absoluteValue
import kotlin.math.cos
import kotlin.math.pow
import kotlin.math.sin

open class MatterGaugePanel<S : Screen> @JvmOverloads constructor(
	screen: S,
	parent: EditablePanel<*>? = null,
	val widget: LevelGaugeWidget,
	x: Float = 0f,
	y: Float = 0f
): EditablePanel<S>(screen, parent, x, y, width = GAUGE_BACKGROUND.width, height = GAUGE_BACKGROUND.height) {
	init {
		scissor = true
	}

	private var wavesStrength = 0.5f
	private var lastAbsoluteX = 0f
	private var lastAbsoluteY = 0f
	private var lastDraw = System.nanoTime()
	private var lastLevel = 0f

	protected open fun makeTooltip(): MutableList<Component> {
		return mutableListOf(
			TranslatableComponent(
				"otm.gui.matter.percentage_level",
				String.format("%.2f", widget.percentage() * 100.0)
			),
			formatMatterLevel(widget.level(), widget.maxLevel())
		)
	}

	override fun innerRender(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float) {
		if (lastAbsoluteX == 0f && lastAbsoluteY == 0f) {
			lastAbsoluteX = absoluteX
			lastAbsoluteY = absoluteY
			lastDraw = System.nanoTime()
			lastLevel = widget.percentage()
		} else {
			val diff = ((lastAbsoluteX - absoluteX).pow(2f) + (lastAbsoluteY - absoluteY).pow(2f)).pow(0.5f) / 10f + (lastLevel - widget.percentage()).absoluteValue * 1.5f

			lastAbsoluteX = absoluteX
			lastAbsoluteY = absoluteY

			wavesStrength = (wavesStrength + diff - (System.nanoTime() - lastDraw) / 400_000_000f).coerceAtLeast(0.5f).coerceAtMost(16f)
			lastDraw = System.nanoTime()
			lastLevel = widget.percentage()
		}

		GAUGE_BACKGROUND.render(stack)
		val height = this.height * (1f - widget.percentage())

		if (widget.percentage() > 0.01f) {
			RenderSystem.setShader(GameRenderer::getPositionTexShader)
			RenderSystem.enableTexture()
			RenderSystem.enableBlend()
			RenderSystem.defaultBlendFunc()
			RenderSystem.depthFunc(GL11.GL_ALWAYS)

			RenderSystem.setShaderColor(1f, 1f, 1f, 1f)

			val u0 = GAUGE_FOREGROUND.x / GAUGE_FOREGROUND.imageWidth
			val u1 = (GAUGE_FOREGROUND.x + GAUGE_FOREGROUND.width) / GAUGE_FOREGROUND.imageWidth
			val v1 = (GAUGE_FOREGROUND.y + GAUGE_FOREGROUND.height) / GAUGE_FOREGROUND.imageHeight

			val matrix = stack.last().pose()
			val builder = tesselator.builder

			builder.begin(VertexFormat.Mode.TRIANGLE_FAN, DefaultVertexFormat.POSITION_TEX)

			builder.vertex(matrix, 0f, this.height, 0f).uv(u0, v1).endVertex()
			builder.vertex(matrix, width, this.height, 0f).uv(u1, v1).endVertex()

			for (i in 4 downTo 0) {
				val sin = sin((System.currentTimeMillis() / 50L + i * 2L).toDouble() / 4.0).toFloat() * wavesStrength.coerceAtMost(4f)
				val cos = cos((System.currentTimeMillis() / 50L + i * 3L + 27L).toDouble() / 4.0).toFloat() * wavesStrength.coerceAtMost(4f)

				val thisX = (width * (i / 4f))
				val thisY = (height + sin + cos).coerceAtLeast(0f)

				builder.vertex(matrix, thisX, thisY, 0f).uv(
					(GAUGE_FOREGROUND.x + (i / 4f) * GAUGE_FOREGROUND.width) / GAUGE_FOREGROUND.imageWidth,
					(GAUGE_FOREGROUND.y + thisY) / GAUGE_FOREGROUND.imageHeight,
				).endVertex()
			}

			BufferUploader.drawWithShader(builder.end())
		}
	}

	override fun innerRenderTooltips(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float): Boolean {
		if (isHovered) {
			screen.renderComponentTooltip(stack, makeTooltip(), mouseX.toInt(), mouseY.toInt())
			return true
		}

		return false
	}

	companion object {
		val GAUGE_BACKGROUND = WidgetLocation.WIDGETS.element(x = 184f, y = 0f, width = 9f, height = 48f)
		val GAUGE_FOREGROUND = WidgetLocation.WIDGETS.element(x = 193f, y = 0f, width = 9f, height = 48f)
	}
}