package ru.dbotthepony.mc.otm.core

import java.math.BigDecimal
import java.math.BigInteger

inline val BigInteger.isZero get() = this == BigInteger.ZERO
inline val BigInteger.isPositive get() = this > BigInteger.ZERO
inline val BigInteger.isNegative get() = this < BigInteger.ZERO

inline val BigDecimal.isZero get() = this == BigDecimal.ZERO
inline val BigDecimal.isPositive get() = this > BigDecimal.ZERO
inline val BigDecimal.isNegative get() = this < BigDecimal.ZERO

val BI_INT_MAX: BigInteger = BigInteger.valueOf(Int.MAX_VALUE.toLong())
val BI_INT_MIN: BigInteger = BigInteger.valueOf(Int.MIN_VALUE.toLong())
val BI_LONG_MAX: BigInteger = BigInteger.valueOf(Long.MAX_VALUE)
val BI_LONG_MIN: BigInteger = BigInteger.valueOf(Long.MIN_VALUE)

val BI_FLOAT_MAX: BigInteger = BigDecimal(Float.MAX_VALUE.toString()).toBigInteger()
val BI_FLOAT_MIN: BigInteger = BigDecimal(Float.MIN_VALUE.toString()).toBigInteger()
val BI_DOUBLE_MAX: BigInteger = BigDecimal(Double.MAX_VALUE.toString()).toBigInteger()
val BI_DOUBLE_MIN: BigInteger = BigDecimal(Double.MIN_VALUE.toString()).toBigInteger()

fun BigInteger.toIntSafe(): Int {
	if (this > BI_INT_MAX) {
		return Int.MAX_VALUE
	} else if (this < BI_INT_MIN) {
		return Int.MIN_VALUE
	}

	return toInt()
}

@Suppress("SameParameterValue")
fun maxScrollDivision(elements: Int, elementsPerRow: Int): Int {
	if (elements % elementsPerRow == 0) {
		return elements / elementsPerRow - 1
	}

	return elements / elementsPerRow
}
