package ru.dbotthepony.mc.otm.core

import org.apache.logging.log4j.LogManager
import java.util.*
import kotlin.ConcurrentModificationException

class TickList {
	private val conditional = LinkedList<IConditionalTickable>()
	private val once = LinkedList<ITickable>()

	private val conditionalValveTime = LinkedList<IConditionalTickable>()
	private val onceValveTime = LinkedList<ITickable>()

	private var inTicker = false

	fun add(ticker: IConditionalTickable) {
		if (inTicker) {
			conditionalValveTime.add(ticker)
		} else {
			conditional.addFirst(ticker)
		}
	}

	fun add(ticker: ITickable) {
		if (inTicker) {
			onceValveTime.add(ticker)
		} else {
			once.addFirst(ticker)
		}
	}

	fun add(ticker: IConditionalTickable, condition: Boolean, reason: String) {
		if (!condition) {
			LOGGER.error("Refusing to add tickable $ticker because we $reason", IllegalStateException(reason))
			return
		}

		return add(ticker)
	}

	fun add(ticker: ITickable, condition: Boolean, reason: String) {
		if (!condition) {
			LOGGER.error("Refusing to add tickable $ticker because we $reason", IllegalStateException(reason))
			return
		}

		return add(ticker)
	}

	fun until(ticker: () -> Boolean) = add(IConditionalTickable.wrap(ticker))
	fun `while`(tickerCondition: () -> Boolean, ticker: () -> Unit) = add(IConditionalTickable.wrap(tickerCondition, ticker))

	fun until(ticker: () -> Boolean, condition: Boolean, reason: String) = add(IConditionalTickable.wrap(ticker), condition, reason)
	fun `while`(tickerCondition: () -> Boolean, ticker: () -> Unit, condition: Boolean, reason: String) = add(IConditionalTickable.wrap(tickerCondition, ticker), condition, reason)

	fun tick() {
		if (inTicker) {
			throw ConcurrentModificationException("Already ticking")
		}

		inTicker = true
		val iterator = conditional.iterator()

		for (ticker in iterator) {
			if (!ticker.canTick) {
				iterator.remove()
			} else {
				ticker.tick()
			}
		}

		for (ticker in once) {
			ticker.tick()
		}

		once.clear()

		for (ticker in conditionalValveTime) {
			conditional.addFirst(ticker)
		}

		for (ticker in onceValveTime) {
			once.addFirst(ticker)
		}

		conditionalValveTime.clear()
		onceValveTime.clear()

		inTicker = false
	}

	fun clear() {
		conditional.clear()
		once.clear()
	}

	companion object {
		private val LOGGER = LogManager.getLogger()
	}
}

fun interface ITickable {
	fun tick()
}

interface IConditionalTickable : ITickable {
	/**
	 * Once this returns false, it should stay false.
	 *
	 * If it suddenly turns true after being false, result is undefined.
	 */
	val canTick: Boolean

	companion object {
		fun wrap(ticker: () -> Boolean): IConditionalTickable {
			return object : IConditionalTickable {
				override var canTick: Boolean = true
					private set

				override fun tick() {
					canTick = !ticker.invoke()
				}
			}
		}

		fun wrap(condition: () -> Boolean, ticker: () -> Unit): IConditionalTickable {
			return object : IConditionalTickable {
				override val canTick: Boolean get() = condition.invoke()

				override fun tick() {
					ticker.invoke()
				}
			}
		}
	}
}
